/*----------------------------------------------------------------------*/
/*
* File: clientm.c
*
* PC demo program for the basic TCP library. Tests connection and data
* transfer with multiple client sockets.
*/
/*----------------------------------------------------------------------*/

#include <conio.h>
#include <stdio.h>

#include "tcp_lib.h"
#include "conn_mul.c"

/*----------------------------------------------------------------------*/
/*                                  Macros                              */
/*----------------------------------------------------------------------*/

/* DDD */
#define DUMP_MDEMO_ERRORS    1  /* dump errors                          */
#define WAIT_EACH_ITER     250  /* wait iteration time (ms), 0=no wait  */
#define DUMP_DATA_SOCKETS    0  /* dump data socket parms               */
#define DUMP_CAMERA_ECHO     0  /* dump camera diag messages            */
                                /* WARNING: This slows down xfer speed! */
#define FLUSH_INTERFACE      0  /* flush interface bef & after I/O  */

/*............. Rasni */
#define DATA_SOCK_CNT        4  /* # of data sockets to connect         */
#define BUF_SIZE             8  /* size of I/O buffers                  */
#define SERVER_PORT       2000  /* server listen port                   */
#define SERVER_IP_ADDR    "192.168.0.65"  /* server IP address (camera) */

/*............. Timeouts */
#define CONN_TIME_OUT        3000   /* connection wait time in ms       */
#define RECV_TIME_OUT         700   /* receive timeout (ms)             */
#define CAM_ECHO_TIMEOUT       50   /* camera echo timeout              */

/*............. Key codes */
#define KEY_F1        0x3B00
#define KEY_F2        0x3C00
#define KEY_F3        0x3D00
#define KEY_F4        0x3E00
#define KEY_F5        0x3F00
#define KEY_F6        0x4000
#define KEY_F7        0x4100
#define KEY_F8        0x4200
#define KEY_F9        0x4300
#define KEY_F10       0x4400
#define KEY_ESC       0x001B

/*----------------------------------------------------------------------*/
/*                             Local functions                          */
/*----------------------------------------------------------------------*/
int  mdemo (
    TCP_SOCK   *sock_23,        /* input socket for port 23             */
    char       *ip_str );       /* input server IP address              */
int get_key ();
void flush_tcp (
    TCP_SOCK   *sock,           /* input data socket                    */
    int         mode );         /* input mode: 0=no dump, 1=dump        */

/****** main = Main Demo Function = 11.2004 *****************************/
/*
* The main test function is a Windows console application. It is a simple
* TCP terminal, which is used to communicate with the camera shell to start
* camera programs and to call the actual I/O function mdemo().
*/
int main(int argc, char *argv[])
{
    int  iret;
    int  key = 0;
    TCP_SOCK sock;        // telnet port 23
    char ip_str[32] = SERVER_IP_ADDR;

/*............. Get new IP address */
    if(argc >= 2)
    {
      strcpy(ip_str,argv[1]);      // save IP address
    }

/*............. Greet user */
    printf("----------------------------------------------------------------\n");
    printf("Demo program with multiple client sockets\n");
    printf("----------------------------------------------------------------\n");
    printf("1. Press Enter several times to see the camera shell prompt '$'.\n");
    printf("2. Press F1 to start test. You must receive in endless loop\n");
    printf("   OK messages for sent and and received data.\n");
    printf("3. Press Q to disconnect camera or other key to disconnect PC\n");
    printf("   and return to camera shell. Press Esc or F10 to exit demo.\n");
    printf("---------------------------------------------------------------\n");

/*............. Init TCP/IP */
    iret = TCP_Init();
    if(iret)
    {
       printf("PC: TCP init error = %d\n",iret);
       return iret;
    }

/*............. Reset data sockets */
    TCP_DataSockReset(&sock,1);

/*............. Connect to default port 23 (camera shell) */
    iret = TCP_Connect(&sock,ip_str,23,CONN_TIME_OUT);
    if(iret)
    {
       printf("PC: TCP connection error = %d\n",iret);
       goto done;
    }
//    printf("PC: Connection to telnet port 23 OK\n");

/*............. Read camera output (if any) */
    TCP_ReadEcho(&sock,1,CAM_ECHO_TIMEOUT);

/*----------------------------------------------------------------------*/
/*                               Main loop                              */
/*----------------------------------------------------------------------*/
    for(;;)
    {
      if(_kbhit())      // key present
      {
        key = get_key();
//printf("key = %02x\n",key);

/*............. Esc = Exit */
        if(key == KEY_ESC ||
           key == KEY_F10 ) break;

/*............. F1 = start mdemo() */
/* Esc : Exit mdemo */
        if(key == KEY_F1)
        {
            mdemo(&sock,ip_str);
            continue;
        }

/*............. Send key to camera */
        if(key == 0x0D) key = 0x0A;
        iret = TCP_SendByte(&sock,(char)key);
        if(iret) printf("Send byte error\n");

      } /* if(_kbhit()) : key present */

/*............. Receive camera bytes and dump */
      while((iret = TCP_RecvByte(&sock)) >= 0)
      {
        if(iret >= 0x20 || iret==0x0A || iret==0x0D || iret==0x08)
          printf("%c",iret);
      }

    } /* for(;;) : main loop */

/*............. Exit */
done:
    TCP_Disconnect(&sock);
    TCP_Close();
    return 0;
}

/****** mdemo = I/O demo = 11.2004 **************************************/
/*----------------------------------------------------------------------*/
/*
* This function communicates with the camera program SERVERM.C. The
* function connects several client sockets and then sends and
* receives data trough the connected sockets in an endless loop.
*
* Parameters:
*   sock_23 = Input telnet socket
*   ip_str  = Input server IP address (the camera)
*
* Return code:
*   0     = OK
*   Other = Errors
*
* Created:  Plamen Kozhuharov, 18.11.2004
*/
/*----------------------------------------------------------------------*/
int  mdemo (
    TCP_SOCK   *sock_23,        /* input socket for port 23             */
    char       *ip_str )        /* input server IP address              */
{

    int      rc = TCP_OK;
    int      iter;
    int      iret;
    int      i,j;
    int      err;
    int      key = 0;
    char     cmd_str[64] = "serverm";  /* camera program name   */

    unsigned char buf[BUF_SIZE];       /* send buffer           */
    unsigned char buf1[BUF_SIZE];      /* receive buffer        */
    TCP_SOCK DataSock[DATA_SOCK_CNT];  /* data socket buffer    */


/*............. Reset all data sockets */
    TCP_DataSockReset(DataSock,DATA_SOCK_CNT);

/*----------------------------------------------------------------------*/
/*                         Start camera program                         */
/*----------------------------------------------------------------------*/
    rc = TCP_SendCmd(sock_23,cmd_str,1);
    if(rc)
    {
#if DUMP_MDEMO_ERRORS
       printf("PC: Camera program start error = %d\n",rc);
#endif
       goto done;
    }

    Sleep(300);
    printf("PC: Camera program started OK\n");

/*----------------------------------------------------------------------*/
/*                     Connect multiple client sockets                  */
/*----------------------------------------------------------------------*/
/* NOTE: The camera server program should be already started ! */

/*............. Connect all sockets to camera */
    rc = TCP_ConnectMul(DataSock,DATA_SOCK_CNT,ip_str,SERVER_PORT,CONN_TIME_OUT);
    if(rc)
    {
#if DUMP_MDEMO_ERRORS
        printf("PC: TCP_ConnectMul error=%d\n",rc);
#endif
        goto done;
    }
    printf("PC: Client's connection OK\n");

    Sleep(150);
    TCP_ReadEcho(sock_23,1,CAM_ECHO_TIMEOUT);  // dump camera messages (if any)

#if DUMP_DATA_SOCKETS
{
  SOCKADDR addr_in;
  sockaddr_in *paddr=(sockaddr_in *)&addr_in;
  int addr_len = sizeof (addr_in);

  printf("PC: --- Data sockets: ---\n");
  for(i=0; i<DATA_SOCK_CNT; i++)
  {
    iret = getsockname(DataSock[i].socket, &addr_in, &addr_len);
    if(iret != 0)
    {
       printf("PC: getsockname error=%d\n",iret);
       goto done;
    }
    printf("    sock=%d: local port=%d (==%d)\n",
               i,ntohs(paddr->sin_port),DataSock[i].port);

//    printf("%d: 0x%x\n",i,DataSock[i].socket);
  }
}
#endif

/*............. Flush interface - read any available data on all sockets */
#if FLUSH_INTERFACE /* [[[ */
    for(i=0; i<DATA_SOCK_CNT; i++)
    {
      flush_tcp(&DataSock[i],1);
    }
#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                            Main iteration loop                       */
/*----------------------------------------------------------------------*/
    iter = 0;               /* number of iterations             */
    for(;;)
    {

/*............. Exit mdemo() on any pressed key */
      if(_kbhit())
      {
        key = _getch();
        if(key == 'Q' || key == 'q')    /* stop camera program */
        {
            TCP_SendByte(sock_23,'x');
        }
        else           /* stop PC demo */
        {
           break;
        }
      }

#if WAIT_EACH_ITER
      Sleep(WAIT_EACH_ITER);
#endif

      printf("PC: --- Iter = %d ---\n",iter);

/*............. Inner loop: Send and receive data on current socket */
      for(i=0; i<DATA_SOCK_CNT; i++)
      {

/* Fill send buffer, clear receive buffer */
        for(j=0; j<BUF_SIZE; j++)
        {
            buf[j] = (i+1)*10 + j;
            buf1[j] = 0;
        }

/* Send block */
        iret = TCP_SendBlock(&DataSock[i],buf,BUF_SIZE);
        if(iret != BUF_SIZE)
        {
           printf("PC: TCP_SendBlock error: isock=%d iret=%d buf_size=%d\n",
                       i,iret,BUF_SIZE);
           rc = TCP_SEND_BLOCK_ERROR;
           goto done;
        }

/* Receive block */
        iret = TCP_RecvBlock(&DataSock[i],buf1,BUF_SIZE,RECV_TIME_OUT);
        if(iret != BUF_SIZE)
        {
           printf("PC: TCP_RecvBlock error: isock=%d iret=%d buf_size=%d\n",
                       i,iret,BUF_SIZE);
           rc = TCP_RECV_BLOCK_ERROR;
           goto done;
        }

/* Check received data */
        err = 0;
        for(j=0; j<BUF_SIZE; j++)
        {
           if(buf[j] != buf1[j]) err++;
        }
        if(err)
        {
           printf("PC: isock=%d: Data error(s) = %d\n",i,err);
        }
        else
        {
           printf("PC: isock=%d: Send/recv block OK\n",i);
        }

      } /* for(i=0; i<DATA_SOCK_CNT; i++) */

#if DUMP_CAMERA_ECHO
      TCP_ReadEcho(sock_23,1,CAM_ECHO_TIMEOUT);    // dump camera diag messages (if any)
#endif

      iter++;
//      if(iter >= 3) break;

    } /* for(;;) : main communication loop */


/*............. Exit */
done:
    Sleep(300);     /* wait some time before disconnecting */

#if FLUSH_INTERFACE /* [[[ - flush interface*/
    for(i=0; i<DATA_SOCK_CNT; i++)
    {
      flush_tcp(&DataSock[i],1);
    }
#endif  /* ]]] */

    TCP_DisconnectMul(DataSock,DATA_SOCK_CNT);
    TCP_ReadEcho(sock_23,1,CAM_ECHO_TIMEOUT);    // dump camera diag messages (if any)

    return rc;
}

/****** get_key = Get Extended Key Code = 07.2004 ***********************/
int get_key ()
{
    int key = _getch();
    if(key == 0 || key == 0xE0)
    {
       key = _getch() << 8;
    }
    return key;
}

/****** flush_tcp = Flush TCP data = 11.2004 ****************************/
void flush_tcp (
    TCP_SOCK   *sock,           /* input data socket                    */
    int         mode )          /* input mode: 0=no dump, 1=dump        */
{
    int iret;
    int n;

/*............. Flush interface - read any available data on all sockets */
    n = 0;
    while((iret = TCP_RecvByte(sock)) >= 0)
    {
      n++;
      if(mode)
      {
         printf("%02x ",iret);
         if(!(n%16))
            printf("\n");
      }
    }
    if(n)
      printf("\n---> PC: sock=%d: %d bytes flushed\n",sock->socket,n);
    return;
}
