/*----------------------------------------------------------------------*/
/*
* File: serverm.c
*
* Camera demo program for the basic TCP library. Tests connection and
* data transfer with multiple server sockets.
*/
/*----------------------------------------------------------------------*/

//#include <stdlib.h>
#include <sysvar.h>
#include <vcrt.h>

#include "tcp_lib.h"
#include "conn_mul.c"


/*----------------------------------------------------------------------*/
/*                                  Macros                              */
/*----------------------------------------------------------------------*/

/* DDD */
#define DUMP_MDEMO_ERRORS     1 /* dump errors                          */
#define TRACE_MDEMO           0 /* trace mdemo execution                */

/*............. Rasni */
#define DATA_SOCK_CNT        4  /* # of data sockets to connect         */
#define BUF_SIZE             8  /* size of I/O buffers                  */
#define SERVER_PORT       2000  /* server listen port                   */

/*............. Timeouts */
#define CONN_TIME_OUT        2000   /* connection wait time in ms       */
#define RECV_TIME_OUT         700   /* recv timeout (ms)                */

/*----------------------------------------------------------------------*/
/*                            Local functions                           */
/*----------------------------------------------------------------------*/
int  exec_mdemo ();
void tfree ();

/*----------------------------------------------------------------------*/
/*                              Global data                             */
/*----------------------------------------------------------------------*/
    TCP_SOCK ListenSock;                /* listen socket                */
    TCP_SOCK DataSock[DATA_SOCK_CNT];   /* data socket buffer           */


/****** main = Main Demo Function = 10.2004 *****************************/
/*
* This camera program communicates with the PC program CLIENTM.C.
* The program accepts connection requests from several remote clients
* and transfers data via multiple server sockets.
*/
int main()
{
    int  rc;
    unsigned long time;

#if TRACE_MDEMO
    printf("Cam: Started...\n");
    tfree();
#endif

/*............. Init TCP */
    rc = TCP_Init();
    if(rc)
    {
#if DUMP_MDEMO_ERRORS
         print("Cam: TCP_Init failed, rc=%d\n",rc);
#endif
         return rc;
    }

/*............. Reset all sockets */
    TCP_DataSockReset(&ListenSock,1);
    TCP_DataSockReset(DataSock,DATA_SOCK_CNT);

/*----------------------------------------------------------------------*/
/*                    Connect multiple server sockets                   */
/*----------------------------------------------------------------------*/

/*............. Create listen socket and and start listening */
    rc = TCP_Listen(&ListenSock,SERVER_PORT);
    if(rc)
    {
#if DUMP_MDEMO_ERRORS
       print("Cam: TCP_Listen failed, rc=%d\n",rc);
#endif
       return rc;
    }
#if TRACE_MDEMO
    printf("Cam: Listen OK\n");
#endif

/*............. Wait for connection */
    rc = TCP_AcceptMul(&ListenSock,DataSock,DATA_SOCK_CNT,CONN_TIME_OUT);
    if(rc != TCP_OK)
    {
       printf("Cam: TCP_WaitConnectMul error=%d\n",rc);
       goto done;
    }

#if TRACE_MDEMO
    printf("Cam: Server connection OK\n");
#endif

/*----------------------------------------------------------------------*/
/*                                 Do I/O                               */
/*----------------------------------------------------------------------*/
    rc = exec_mdemo();

/*............. Exit */
done:
    time = get_time();        /* wait some time before disconnect */
    while(get_time() - time < 300);

    TCP_DisconnectMul(DataSock,DATA_SOCK_CNT);
    TCP_Disconnect(&ListenSock);
    TCP_Close();

#if TRACE_MDEMO
    printf("Cam: Exit, rc=%d\n",rc);
    tfree();
#endif
    return rc;

}

/****** exec_mdemo = Execute I/O demo = 11.2004 *************************/
/*----------------------------------------------------------------------*/
/*
* This function communicates with the PC program CLIENTM.C. The
* function connects several server sockets and then receives amd sends
* back data trough connected sockets in an endless loop.
*
* Parameters:
*   None
*
* Return code:
*   0     = OK
*   Other = Errors
*
* Created:  Plamen Kozhuharov, 21.11.2004
*/
/*----------------------------------------------------------------------*/
int  exec_mdemo ()
{

    int      rc = TCP_OK;
    unsigned char buf[BUF_SIZE];       /* I/O buffer            */
    int      iter;
    int      i;
    int      iret;
//    unsigned long time;

/*----------------------------------------------------------------------*/
/*                            Main iteration loop                       */
/*----------------------------------------------------------------------*/
#if TRACE_MDEMO
    printf("Cam: Starting iteration loop...\n");
#endif

    iter = 0;               /* number of iterations             */
    for(;;)
    {

/*............. Exit program on any received byte on telnet port 23 */
      if(rbempty() != -1)
      {
#if TRACE_MDEMO
        printf("Cam: Exit on telnet input\n");
#endif
        rs232rcv();     // consume the byte
        break;
      }

#if TRACE_MDEMO
      printf("Cam: --- Iter = %d ---\n",iter);
#endif

/*............. Inner loop: Send and receive data on current socket */
      for(i=0; i<DATA_SOCK_CNT; i++)
      {

/*............. Receive block */
        iret = TCP_RecvBlock(&DataSock[i],buf,BUF_SIZE,RECV_TIME_OUT);
        if(iret != BUF_SIZE)
        {
#if DUMP_MDEMO_ERRORS
           printf("Cam: TCP_RecvBlock error: isock=%d iret=%d buf_size=%d\n",
                       i,iret,BUF_SIZE);
#endif
           rc = TCP_RECV_BLOCK_ERROR;
           goto done;
        }

/*............. Send block */
        iret = TCP_SendBlock(&DataSock[i],buf,BUF_SIZE);
        if(iret != BUF_SIZE)
        {
#if DUMP_MDEMO_ERRORS
           printf("Cam: TCP_SendBlock error: isock=%d iret=%d buf_size=%d\n",
                       i,iret,BUF_SIZE);
#endif
           rc = TCP_SEND_BLOCK_ERROR;
           goto done;
        }

      } /* for(i=0; i<DATA_SOCK_CNT; i++) */

      iter++;
//      if(iter >= 2) break;

    } /* for(;;) : main communication loop */

/*............. Exit */
done:
    return rc;
}

/****** tfree = Print start free heap address = 11.2004 *****************/
void tfree ()
{
    int *buf = vcmalloc(1);
    if(!buf)
    {
      print("Cam: tfree allocation error\n");
      return;
    }
    printf("Cam: Heap = %08x\n",(int)buf);

    vcfree(buf);
    return;
}
