/*----------------------------------------------------------------------*/
/*
* File: client.c
*
* Demo program for TCP client. The program attempts to connect to a server,
* specified by IP address and port. After successful connection the
* program sends a block of 8 bytes to the server and receives back a block
* of 8 bytes.
*
* Note: The program uses the default IP address of the camera 192.168.0.65.
*       Change if necessary !
*
* _TI_CAMERA defined     : Compile for camera
* _TI_CAMERA not defined : Compile for PC
*/
/*----------------------------------------------------------------------*/

#ifdef _TI_CAMERA
  #include <vcrt.h>
#else
  #include <stdio.h>
#endif
#include "tcp_lib.h"

#define CONN_TIME_OUT   2000            /* connection wait time in ms   */
#define RECV_TIME_OUT    500            /* receive timeout in ms        */
#define BUF_SIZE           8            /* I/O buffer size              */
#define SERVER_IP_ADDR  "192.168.0.65"  /* server IP address            */
#define SERVER_PORT     2000            /* server listen port           */

int main(int argc, char *argv[])
{

    int  rc;
    int  iret;
    unsigned char buf[BUF_SIZE]  = { 0,1,2,3,4,5,6,7 }; /* send buffer  */
    unsigned char buf1[BUF_SIZE] = { 0,0,0,0,0,0,0,0 }; /* recv buffer  */
    char ip_str[32] = SERVER_IP_ADDR;

    TCP_SOCK DataSock;              /* data socket              */

/*............. Get new IP address */
    if(argc >= 2)
    {
      strcpy(ip_str,argv[1]);      // save IP address
    }

/*............. Init TCP */
    rc = TCP_Init();
    if(rc != TCP_OK)
    {
        printf("TCP_Init error = %d\n",rc);
        return rc;
    }

/*............. Reset socket(s) */
    TCP_DataSockReset(&DataSock,1);

/*............. Connect to a remote server */
    rc = TCP_Connect(&DataSock,ip_str,SERVER_PORT,CONN_TIME_OUT);
    if(rc != TCP_OK) goto done;

/*............. Send block */
    iret = TCP_SendBlock(&DataSock,buf,BUF_SIZE);
    if(iret != BUF_SIZE)
    {
        printf("TCP_SendBlock error: iret=%d buf_size=%d\n",iret,BUF_SIZE);
        rc = TCP_SEND_BLOCK_ERROR;
        goto done;
    }

/*............. Receive block */
    iret = TCP_RecvBlock(&DataSock,buf1,BUF_SIZE,RECV_TIME_OUT);
    if(iret != BUF_SIZE)
    {
        printf("TCP_RecvBlock error: iret=%d buf_size=%d\n",iret,BUF_SIZE);
        rc = TCP_RECV_BLOCK_ERROR;
        goto done;
    }
    else
    {
        printf("recv: %d %d %d %d %d %d %d %d\n",
            buf1[0],buf1[1],buf1[2],buf1[3],
            buf1[4],buf1[5],buf1[6],buf1[7]);
    }

/*............. Exit */
done:
    TCP_Disconnect(&DataSock);
    TCP_Close();

    printf("RC = %d\n",rc);
    return rc;
}
