/*----------------------------------------------------------------------*/
/*
* File: server.c
*
* Demo program for TCP server. The program waits for connection request
* from a remote client on port 2000. After successful connection the
* program receives a block of 8 bytes and sends back the block to the
* client.
*
* _TI_CAMERA defined     : Compile for camera
* _TI_CAMERA not defined : Compile for PC
*/
/*----------------------------------------------------------------------*/

#ifdef _TI_CAMERA
  #include <vcrt.h>
#else
  #include <stdio.h>
  #include <conio.h>
#endif
#include "tcp_lib.h"

#define CONN_TIME_OUT   9000        /* connection wait time in ms   */
#define RECV_TIME_OUT    500        /* receive timeout in ms        */
#define BUF_SIZE           8        /* I/O buffer size              */
#define SERVER_PORT     2000        /* server listen port           */

int main()
{

    int  rc;
    int  iret;
    unsigned long time;
    unsigned char buf[BUF_SIZE];    /* I/O buffer               */

    TCP_SOCK ListenSock;            /* listen socket            */
    TCP_SOCK DataSock;              /* data socket              */

/*............. Init TCP */
    rc = TCP_Init();
    if(rc != TCP_OK)
    {
        printf("TCP_Init error = %d\n",rc);
        return rc;
    }

/*............. Reset sockets */
    TCP_DataSockReset(&ListenSock,1);
    TCP_DataSockReset(&DataSock,1);

/*............. Create listen socket and start listening on port 2000 */
    rc = TCP_Listen(&ListenSock,SERVER_PORT);
    if(rc != TCP_OK) goto done;

/*............. Wait for connection with a remote client */
    time = get_time();
    for(;;)
    {
        rc = TCP_AcceptConnection(&ListenSock,&DataSock,1);
        if(rc != TCP_OK) goto done;

        if(TCP_GetConnectionState(&DataSock)) break;    /* connection OK */

        if(get_time() - time >= CONN_TIME_OUT)
        {
           rc = TCP_NO_CONNECTION_ERROR;
           goto done;
        }
    } /* for(;;) */

/*............. Receive block */
    iret = TCP_RecvBlock(&DataSock,buf,BUF_SIZE,RECV_TIME_OUT);
    if(iret != BUF_SIZE)
    {
        printf("TCP_RecvBlock error: iret=%d buf_size=%d\n",iret,BUF_SIZE);
        rc = TCP_RECV_BLOCK_ERROR;
        goto done;
    }

/*............. Send block */
    iret = TCP_SendBlock(&DataSock,buf,BUF_SIZE);
    if(iret != BUF_SIZE)
    {
        printf("TCP_SendBlock error: iret=%d buf_size=%d\n",iret,BUF_SIZE);
        rc = TCP_SEND_BLOCK_ERROR;
        goto done;
    }

/*............. Exit */
done:
    time = get_time();
    while(get_time() - time < 300);     /* wait before disconnect */

    TCP_Disconnect(&DataSock);
    TCP_Disconnect(&ListenSock);
    TCP_Close();

    printf("RC = %d\n",rc);
    return rc;
}
